import 'dart:async';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_tik_video/models/collect_model.dart';
import 'package:eyflutter_tik_video/models/portrait_model.dart';
import 'package:eyflutter_tik_video/models/share_model.dart';
import 'package:eyflutter_tik_video/models/video_loading_model.dart';
import 'package:eyflutter_tik_video/models/video_play_progress_model.dart';
import 'package:eyflutter_tik_video/models/video_player_icon_model.dart';
import 'package:eyflutter_tik_video/tiklist/favorite_animation_icon.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:flutter/material.dart';
import 'package:percent_indicator/percent_indicator.dart';

/// 单击：暂停；双击：点赞，双击后紧接着一次单击也是点赞；
class TikVideoGesture extends StatefulWidget {
  /// 子组件
  final Widget child;

  /// 播放图标单击事件
  final Function(VideoPlayerIconModel model) playIconTap;

  /// 点赞回调
  final Function()? favoriteCall;

  /// 单击回调
  final Function()? singleTapCall;

  /// 构建完成回调
  final Function() buildCall;

  /// 播放进度
  final VideoPlayProgressModel progressModel;

  /// 播放图标
  final VideoPlayerIconModel playerIconModel;

  /// 加载状态
  final VideoLoadingModel loadingModel;

  /// 收藏
  final CollectModel collectModel;

  /// 分享
  final ShareModel shareModel;

  /// 进度条颜色
  final Color progressColor;

  /// 进度条背景
  final Color progressBackgroundColor;

  final Function(PortraitModel model)? portraitCall;

  final Function(CollectModel model)? collectCall;

  final Function(ShareModel model)? shareCall;

  const TikVideoGesture({
    Key? key,
    required this.child,
    required this.playIconTap,
    this.favoriteCall,
    this.singleTapCall,
    required this.buildCall,
    required this.progressModel,
    required this.playerIconModel,
    required this.loadingModel,
    required this.collectModel,
    required this.progressColor,
    required this.progressBackgroundColor,
    required this.shareModel,
    this.portraitCall,
    this.collectCall,
    this.shareCall,
  }) : super(key: key);

  @override
  _TikVideoGestureState createState() => _TikVideoGestureState();
}

class _TikVideoGestureState extends State<TikVideoGesture> {
  List<Offset> icons = [];
  bool canAddFavorite = false;
  bool justAddFavorite = false;
  GlobalKey key = GlobalKey();
  Timer? timer;
  PortraitModel portraitModel = PortraitModel();

  @override
  void initState() {
    super.initState();
    widget.buildCall.call();
    CloudEBus.instance.addEvent(this, action: "playIconSetting", method: onPlayIconSetting);
  }

  @override
  void dispose() {
    super.dispose();
    CloudEBus.instance.dispatch("playIconSetting");
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      child: Stack(
        children: <Widget>[
          widget.child,
          buildFavoriteAnimationView(),
          buildRightView(),
          buildPlayWidget(),
          buildLoadingWidget(),
          Positioned(
            child: buildPlayProgress(),
            left: 0,
            bottom: 0,
            right: 0,
          ),
        ],
      ),
      onTapDown: (detail) {
        setState(() {
          if (canAddFavorite) {
            icons.add(detail.globalPosition);
            widget.favoriteCall?.call();
            justAddFavorite = true;
          } else {
            justAddFavorite = false;
          }
        });
      },
      onTapUp: (detail) {
        timer?.cancel();
        var delay = canAddFavorite ? 500 : 600;
        timer = Timer(Duration(milliseconds: delay), () {
          canAddFavorite = false;
          timer = null;
          if (!justAddFavorite) {
            widget.singleTapCall?.call();
          }
        });
        canAddFavorite = true;
      },
    );
  }

  Widget buildPlayProgress() {
    return StateUtils.instance.subscribeNotifierWidget<VideoPlayProgressModel>(widget.progressModel, (context, model) {
      return LinearPercentIndicator(
        width: MediaUtils.instance.screenWidth,
        lineHeight: 2,
        percent: model.currentProgress,
        backgroundColor: widget.progressBackgroundColor,
        progressColor: widget.progressColor,
        padding: EdgeInsets.zero,
      );
    });
  }

  Widget buildRightView() {
    return Container(
      alignment: Alignment.centerRight,
      child: Padding(
        padding: const EdgeInsets.only(right: 20, bottom: 60),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            buildPortrait(),
            buildCollect(),
            buildShare(),
          ],
        ),
      ),
    );
  }

  Widget buildShare() {
    return StateUtils.instance.subscribeNotifierWidget<ShareModel>(widget.shareModel, (context, model) {
      return Padding(
        padding: const EdgeInsets.only(top: 12),
        child: GestureDetector(
          child: Image.asset(
            "assets/images/share_icon.png",
            package: "eyflutter_tik_video",
            width: 42,
            height: 42,
          ),
          onTap: () => widget.shareCall?.call(model),
        ),
      );
    });
  }

  Widget buildCollect() {
    return StateUtils.instance.subscribeNotifierWidget<CollectModel>(widget.collectModel, (context, model) {
      //这里根据isCollect替换收藏图片
      return Padding(
        padding: const EdgeInsets.only(top: 12),
        child: GestureDetector(
          child: Image.asset(
            "assets/images/love_icon.png",
            package: "eyflutter_tik_video",
            width: 42,
            height: 42,
          ),
          onTap: () => widget.collectCall?.call(model),
        ),
      );
    });
  }

  Widget buildPortrait() {
    return StateUtils.instance.subscribeNotifierWidget<PortraitModel>(portraitModel, (context, model) {
      return Container(
        width: 52,
        height: 52,
        decoration: BoxDecoration(
          border: Border.all(color: Colors.white, width: 0.8),
          borderRadius: const BorderRadius.all(Radius.circular(28)),
          color: Colors.white,
        ),
        child: GestureDetector(
          child: Container(
            width: 42,
            height: 42,
            alignment: Alignment.center,
            child: Image.asset(
              "assets/images/ey_head_icon.png",
              package: "eyflutter_tik_video",
              width: 42,
              height: 42,
            ),
          ),
          onTap: () => widget.portraitCall?.call(model),
        ),
      );
    });
  }

  Widget buildFavoriteAnimationView() {
    return Stack(
      children: icons
          .map<Widget>(
            (p) => FavoriteAnimationIcon(
              key: Key(p.toString()),
              position: p,
              favoriteComplete: () {
                icons.remove(p);
              },
              size: 50,
            ),
          )
          .toList(),
    );
  }

  Widget buildLoadingWidget() {
    return StateUtils.instance.subscribeNotifierWidget<VideoLoadingModel>(widget.loadingModel, (context, model) {
      return Visibility(
        child: const Align(
          alignment: Alignment.center,
          child: CircularProgressIndicator(
            strokeWidth: 8,
            backgroundColor: Colors.grey,
            valueColor: AlwaysStoppedAnimation<Color>(Colors.blueGrey),
          ),
        ),
        visible: model.isLoading,
      );
    });
  }

  Widget buildPlayWidget() {
    return StateUtils.instance.subscribeNotifierWidget<VideoPlayerIconModel>(widget.playerIconModel, (context, model) {
      return Visibility(
        child: Align(
          alignment: Alignment.center,
          child: GestureDetector(
            child: Image.asset(
              "assets/images/ey_playing_icon.png",
              package: "eyflutter_tik_video",
              width: 58,
              height: 58,
            ),
            onTap: () => widget.playIconTap.call(model),
          ),
        ),
        visible: !model.isPlaying,
      );
    });
  }

  void onPlayIconSetting(bool isHidePlayIcon) {
    if (isHidePlayIcon) {
      widget.playerIconModel.isPlaying = true;
    } else {
      widget.playerIconModel.isPlaying = false;
    }
    widget.playerIconModel.notifierLazy();
  }
}
